/***************************************************************************
 *                                                                         *
 *  Copyright 2009 LSI  Corporation.  All rights reserved.                 *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corp.  The         *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Corporation.                                    *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  usbLibDesc.h
 *          Title:  USB library descriptors header.
 *     Programmer:  Sourin Sarkar
 *  Creation Date:  Jan 12, 2010
 *
 *
 * Description
 * ------------
 * This file contains header descriptor definitions for the USB stack / core
 * source.
 *
 * Note:
 *
 *-------------------------------------------------------------------------
 */


#ifndef _USBLIBDESC_H_
#define _USBLIBDESC_H_

#if (IOC_USB_SUPPORT)

/* ************************************************************************** */
/* Descriptor - Device                                                        */
/* ************************************************************************** */

typedef struct _USB_DESCRIPTOR_DEVICE
{
    U8      DescLength;             /* Descriptor size in bytes = 18 */
    U8      DescType;               /* DEVICE descriptor type = 1 */
    U8      UsbVerInBCD[2];         /* USB spec in BCD, e.g. 0x0200 */
    U8      DeviceClass;            /* Class code, if 0 see interface */
    U8      DeviceSubClass;         /* Sub-Class code, 0 if class = 0 */
    U8      DeviceProtocol;         /* Protocol, if 0 see interface */
    U8      MaxPacketSize;          /* Endpoint 0 max. size */
    U8      VendorId[2];            /* Vendor ID per USB-IF */
    U8      ProductId[2];           /* Product ID per manufacturer */
    U8      DeviceRelInBcd[2];      /* Device release # in BCD */
    U8      ManufacturerStrIndex;   /* Index to manufacturer string */
    U8      ProductStrIndex;        /* Index to product string */
    U8      SerialNumStrIndex;      /* Index to serial number string */
    U8      NumConfigs;             /* Number of possible configurations */

} USB_DESCRIPTOR_DEVICE, *PTR_USB_DESCRIPTOR_DEVICE;


/* ************************************************************************** */
/* Descriptor - Configuration                                                 */
/* ************************************************************************** */
#define CONFIG_RES7         (0x80)  /* Reserved, always = 1 */
#define CONFIG_SELF_PWR     (0x40)  /* Self-powered device */
#define CONFIG_WAKEUP       (0x20)  /* Remote wakeup */

typedef struct _USB_DESCRIPTOR_CFG
{
    U8      DescLength;             /* Descriptor size in bytes = 9 */
    U8      DescType;               /* CONFIGURATION type = 2 or 7 */
    U8      TotalLength[2];         /* Length of concatenated descriptors */
    U8      NumInterfaces;          /* Number of interfaces, this config. */
    U8      ConfigurationValue;     /* Value to set this config. */
    U8      ConfigStrIndex;         /* Index to configuration string */
    U8      Attributes;             /* Config. characteristics */
    U8      MaxPower;               /* Max.power from bus, 2mA units */

} USB_DESCRIPTOR_CFG, *PTR_USB_DESCRIPTOR_CFG;


/* ************************************************************************** */
/* Descriptor - Interface                                                     */
/* ************************************************************************** */
typedef struct _USB_DESCRIPTOR_INTERFACE
{
    U8      DescLength;             /* Descriptor size in bytes = 9 */
    U8      DescType;               /* INTERFACE descriptor type = 4 */
    U8      InterfaceNumber;        /* Interface no.*/
    U8      AlternateSetting;       /* Value to select this IF */
    U8      NumEndpoints;           /* Number of endpoints excluding 0 */
    U8      InterfaceClass;         /* Class code, 0xFF = vendor */
    U8      InterfaceSubClass;      /* Sub-Class code, 0 if class = 0 */
    U8      InterfaceProtocol;      /* Protocol, 0xFF = vendor */
    U8      InterfaceStrIndex;      /* Index to interface string */

} USB_DESCRIPTOR_INTERFACE, *PTR_USB_DESCRIPTOR_INTERFACE;


/* ************************************************************************** */
/* Descriptor - End Point                                                     */
/* ************************************************************************** */
#define USB_ENDPOINT_IDX_CONTROL            0   /* Used to index into the array */
#define USB_ENDPOINT_IDX_IN                 1   /* Used to index into the array */
#define USB_ENDPOINT_IDX_OUT                2   /* Used to index into the array */

#define USB_ENDPOINT_DIRECTION_IN           (0x80)  /* IN endpoint, device to host */
#define USB_ENDPOINT_DIRECTION_OUT          (0x00)  /* OUT endpoint, host to device */
#define USB_ENDPOINT_DIRECTION_MASK         (0x0F)  /* Mask endpoint # */

#define USB_ENDPOINT_TYPE_CONTROL           (0x00)  /* Control transfers */
#define USB_ENDPOINT_TYPE_ISOCHRONOUS       (0x01)  /* Isochronous transfers */
#define USB_ENDPOINT_TYPE_BULK              (0x02)  /* Bulk transfers */
#define USB_ENDPOINT_TYPE_INTERRUPT         (0x03)  /* Interrupt transfers */
#define USB_ENDPOINT_TYPE_MASK              (0x03)  /* Mask type bits */

#define USB_ISOCH_NOSYNC                    (0x00)  /* No synchronization */
#define USB_ISOCH_ASYNC                     (0x04)  /* Asynchronous */
#define USB_ISOCH_ADAPT                     (0x08)  /* Adaptive */
#define USB_ISOCH_SYNCH                     (0x0C)  /* Synchrounous */
#define USB_ISOCH_DATA                      (0x00)  /* Data endpoint */
#define USB_ISOCH_FEEDBACK                  (0x10)  /* Feedback endpoint */
#define USB_ISOCH_IMPLICIT                  (0x20)  /* Implicit feedback */
#define USB_ISOCH_RESERVED                  (0x30)  /* Reserved */

#define PACKET_SIZE_MASK                    (0x7FF)     /* packet size bits */
#define NO_ADDITONAL                        (0x0000)    /* 1 / microframe */
#define ONE_ADDITIONAL                      (0x0800)    /* 2 / microframe */
#define TWO_ADDITIONAL                      (0x1000)    /* 3 / microframe */
#define ADDITIONAL_MASK                     (ONE_ADDITIONAL | TWO_ADDITIONAL)

typedef struct _USB_DESCRIPTOR_ENDPOINT
{
    U8      DescLength;             /* Descriptor size in bytes = 7 */
    U8      DescType;               /* ENDPOINT descriptor type = 5 */
    U8      EndpointAddress;        /* Endpoint # 0 - 15 | IN/OUT */
    U8      Attributes;             /* Transfer type */
    /* Following must be zero except for isochronous endpoints */
    U8      MaxPacketSize[2];       /* Bits 10:0 = max. packet size */
    /* For high-speed interrupt or isochronous only, additional
    **   transaction opportunities per microframe follow.*/
    U8      PollInterval;           /* Polling interval in (micro) frames */

} USB_DESCRIPTOR_ENDPOINT, *PTR_USB_DESCRIPTOR_ENDPOINT;


/* ************************************************************************** */
/* Descriptor - Qualifier                                                     */
/* ************************************************************************** */
typedef struct _USB_DESCRIPTOR_QUALIFIER
{
    U8      DescLength;             /* Descriptor size in bytes = 10 */
    U8      DescType;               /* DEVICE QUALIFIER type = 6 */
    U8      bcdUSD[2];              /* USB spec in BCD, e.g. 0x0200 */
    U8      DeviceClass;            /* Class code, if 0 see interface */
    U8      DeviceSubClass;         /* Sub-Class code, 0 if class = 0 */
    U8      DeviceProtocol;         /* Protocol, if 0 see interface */
    U8      MaxPacketSize;          /* Endpoint 0 max. size */
    U8      NumConfigurations;      /* Number of possible configurations */
    U8      Reserved;               /* Reserved = 0 */

} USB_DESCRIPTOR_QUALIFIER, *PTR_USB_DESCRIPTOR_QUALIFIER;


/* Other-Config type 7 fields are identical to type 2 above */

/* Interface-Power descriptor  type 8 not used  in this version */

/* ************************************************************************** */
/* Descriptor - OTG                                                           */
/* ************************************************************************** */
#define OTG_SRP_SUPPORT     (0x01)  /* Supports SRP */
#define OTG_HNP_SUPPORT     (0x02)  /* Supports HNP */

typedef struct _USB_DESCRIPTOR_OTG
{
    U8      DescLength;             /* Descriptor size in bytes = 9 */
    U8      DescType;               /* CONFIGURATION type = 2 or 7 */
    U8      Attributes;             /* OTG characteristics */

} USB_DESCRIPTOR_OTG, *PTR_USB_DESCRIPTOR_OTG;


/* ************************************************************************** */
/* Descriptor - Union                                                         */
/* ************************************************************************** */
typedef union _USB_DESCRIPTOR_UNION
{
   PTR_USB_DESCRIPTOR_DEVICE        PtrUsbDescDevice;
   PTR_USB_DESCRIPTOR_CFG           PtrUsbDescCfg;
   PTR_USB_DESCRIPTOR_INTERFACE     PtrUsbDescInterface;
   PTR_USB_DESCRIPTOR_ENDPOINT      PtrUsbDescEndPoint;
   PTR_USB_DESCRIPTOR_QUALIFIER     PtrUsbDescQualifier;
   PTR_USB_DESCRIPTOR_OTG           PtrUsbDescOtg;

   U32                              DWord;
   PU8                              Buffer;
   PU32                             PtrDWord;

} USB_DESCRIPTOR_UNION, *PTR_USB_DESCRIPTOR_UNION;



/* ************************************************************************** */
/*                H O S T    D A T A    S T R U C T U R E S                   */
/* ************************************************************************** */

#define USB_NEXT_LINK_PTR_SELECT_TYPE_ITD                   (0x00)  /* Isochronous Transfer Descriptor (ITD) */
#define USB_NEXT_LINK_PTR_SELECT_TYPE_QUEUE_HEAD            (0x01)  /* Queue Head */
#define USB_NEXT_LINK_PTR_SELECT_TYPE_SPLIT_TX_ITD          (0x02)  /* Split Transaction ITD */
#define USB_NEXT_LINK_PTR_SELECT_TYPE_FSTN                  (0x03)  /* Frame Span Traversal Node */

#define USB_NEXT_LINK_PTR_VALID                             (0x00)  /* 1-bit set to '0' to indicate valid pointer */
#define USB_NEXT_LINK_PTR_INVALID                           (0x01)  /* 1-bit set to '1' to indicate invalid pointer */

/* ************************************************************************** */
/* Frame List Pointer / Next Link Pointer                                     */
/* ************************************************************************** */

typedef struct _USB_NEXT_LINK_POINTER
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     NextLinkPointer : 27;
            U32     Reserved        :  2;
            U32     PointerType     :  2;
            U32     TerminateBit    :  1;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     TerminateBit    :  1;
            U32     PointerType     :  2;
            U32     Reserved        :  2;
            U32     NextLinkPointer : 27;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_NEXT_LINK_POINTER, *PTR_USB_NEXT_LINK_POINTER;


/* ************************************************************************** */
/* iTD Transaction Status and Control List Element                            */
/* ************************************************************************** */
typedef struct _USB_ITD_TX_STATUS_AND_CTRL_LIST_ELEMENT
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Status              :  4;
            U32     TransactionXLength  : 12;
            U32     InterruptOnComplete :  1;
            U32     PageSelect          :  3;
            U32     TransactionXOffset  : 12;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     TransactionXOffset  : 12;
            U32     PageSelect          :  3;
            U32     InterruptOnComplete :  1;
            U32     TransactionXLength  : 12;
            U32     Status              :  4;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_ITD_TX_STATUS_AND_CTRL_LIST_ELEMENT, *PTR_USB_ITD_TX_STATUS_AND_CTRL_LIST_ELEMENT;


/* ************************************************************************** */
/* iTD Buffer Page Pointer List Element (0)                                   */
/* ************************************************************************** */
typedef struct _USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_0
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     BufferPointer       : 20;
            U32     EndPointNumber      :  4;
            U32     Reserved07          :  1;
            U32     DeviceAddress       :  7;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     DeviceAddress       :  7;
            U32     Reserved07          :  1;
            U32     EndPointNumber      :  4;
            U32     BufferPointer       : 20;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_0, *PTR_USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_0;


/* ************************************************************************** */
/* iTD Buffer Page Pointer List Element (1)                                   */
/* ************************************************************************** */
typedef struct _USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_1
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     BufferPointer       : 20;
            U32     Direction           :  1;
            U32     MaximumPacketSize   : 11;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     MaximumPacketSize   : 11;
            U32     Direction           :  1;
            U32     BufferPointer       : 20;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_1, *PTR_USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_1;


/* ************************************************************************** */
/* iTD Buffer Page Pointer List Element (2)                                   */
/* ************************************************************************** */
#define USB_ITD_BUFF_PAGE_MULTI_VAL_RESERVED                        (0x00) /* Results undefined */
#define USB_ITD_BUFF_PAGE_MULTI_VAL_1_TX_FOR_EP_PER_MFRAME          (0x01) /* 1 transaction to be issued for EP per micro frame */
#define USB_ITD_BUFF_PAGE_MULTI_VAL_2_TX_FOR_EP_PER_MFRAME          (0x02) /* 2 transaction to be issued for EP per micro frame */
#define USB_ITD_BUFF_PAGE_MULTI_VAL_3_TX_FOR_EP_PER_MFRAME          (0x03) /* 3 transaction to be issued for EP per micro frame */

typedef struct _USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_2
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     BufferPointer       : 20;
            U32     Reserved            : 10;
            U32     Multi               :  2;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     Multi               :  2;
            U32     Reserved            : 10;
            U32     BufferPointer       : 20;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_2, *PTR_USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_2;


/* ************************************************************************** */
/* iTD Buffer Page Pointer List Element (3 - 6)                               */
/* ************************************************************************** */
typedef struct _USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_3_6
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     BufferPointer       : 20;
            U32     Reserved00to11      : 12;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     Reserved00to11      : 12;
            U32     BufferPointer       : 20;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_3_6, *PTR_USB_ITD_BUFFER_PAGE_PTR_LIST_ELEMENT_3_6;


/* ************************************************************************** */
/* Split TX iTD End Point Capabilities / Characteristics                      */
/* ************************************************************************** */
typedef struct _USB_SPLIT_TX_ITD_EP_CAPABILITIES
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Direction           : 1;
            U32     PortNumber          : 7;
            U32     Reserved23          : 1;
            U32     HubAddress          : 7;
            U32     Reserved12to15      : 4;
            U32     EndPointNumber      : 4;
            U32     Reserved07          : 1;
            U32     DeviceAddress       : 7;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     DeviceAddress       : 7;
            U32     Reserved07          : 1;
            U32     EndPointNumber      : 4;
            U32     Reserved12to15      : 4;
            U32     HubAddress          : 7;
            U32     Reserved23          : 1;
            U32     PortNumber          : 7;
            U32     Direction           : 1;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_SPLIT_TX_ITD_EP_CAPABILITIES, *PTR_USB_SPLIT_TX_ITD_EP_CAPABILITIES;


/* ************************************************************************** */
/* Split TX iTD Micro Frame Schedule Control                                  */
/* ************************************************************************** */
typedef struct _USB_SPLIT_TX_ITD_MICRO_FRAME_SCHED_CTRL
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     Reserved16to31      : 16;
            U32     SplitCompletionMask :  8;
            U32     SplitStartMask      :  8;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     SplitStartMask      :  8;
            U32     SplitCompletionMask :  8;
            U32     Reserved16to31      : 16;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_SPLIT_TX_ITD_MICRO_FRAME_SCHED_CTRL, *PTR_USB_SPLIT_TX_ITD_MICRO_FRAME_SCHED_CTRL;


/* ************************************************************************** */
/* Split TX iTD Transfer Status and Control                                   */
/* ************************************************************************** */

/* Set when reqd. to enable execution of Split ITD */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_ACTIVE                   (0x01 << 7)
/* Set when error response received from companion controller */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_ERR                      (0x01 << 6)
/* Set when Host controller indicates data overrun / underrun */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_DATA_BUFF_ERR            (0x01 << 5)
/* Set when a babble is detected */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_BABBLE_DETECTED          (0x01 << 4)
/* Set when not received a proper response from device */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_TRANSACTION_ERR          (0x01 << 3)
/* Set when host induced hold off caused host ctrlr. to miss one complete-split transaction */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_MISSED_MICRO_FRAME       (0x01 << 2)
/* Set when host controller wants to indicate to do a complete split transaction */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_SPLIT_XSTATE             (0x01 << 1)
/* Should be always set to '0' */
#define USB_SPLIT_ITD_TXFER_STATUS_VAL_RESERVED                         (0)

typedef struct _USB_SPLIT_ITD_TXFER_STATUS_AND_CTRL
{
    union
    {
        struct
        {
#if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN )
            U32     InterruptOnCompleteBit          :  1;
            U32     PageSelectIndicatorBit          :  1;
            U32     Reserved26to29                  :  4;
            U32     TotalBytesToTransfer            : 10;
            U32     MicroFrameCompleteSpeedProgMask :  8;
            U32     Status                          :  8;
#else /* if defined( __LITTLE_ENDIAN__ ) */
            U32     Status                          :  8;
            U32     MicroFrameCompleteSpeedProgMask :  8;
            U32     TotalBytesToTransfer            : 10;
            U32     Reserved26to29                  :  4;
            U32     PageSelectIndicatorBit          :  1;
            U32     InterruptOnCompleteBit          :  1;
#endif /* #if defined( __BIG_ENDIAN__ ) || defined( __BIG_ENDIAN ) */
        } Fields;

        U32         DWord;

    } Reg;

} USB_SPLIT_ITD_TXFER_STATUS_AND_CTRL, *PTR_USB_SPLIT_ITD_TXFER_STATUS_AND_CTRL;

#endif /* #if (IOC_USB_SUPPORT) */

#endif /* #ifndef _USBLIBDESC_H_ */

