/*******************************************************************************

NAME            pci.c
VERSION         %version: 2 %
UPDATE DATE     %date_modified: Thu Oct 17 11:32:58 2013 %
PROGRAMMER      %created_by:    weberc %

        Copyright 2013 NetApp, Inc. All rights reserved.

*******************************************************************************/
/***************************************************************************
 *                                                                         *
 *  Copyright 2012 LSI Corporation.  All rights reserved.                  *
 *                                                                         *
 *  This file is confidential and a trade secret of LSI Corporation.  The  *
 *  receipt of or possession of this file does not convey any rights to    *
 *  reproduce or disclose its contents or to manufacture, use, or sell     *
 *  anything it may describe, in whole, or in part, without the specific   *
 *  written consent of LSI Logic Corporation.                              *
 *                                                                         *
 ***************************************************************************
 *
 *           Name:  pci.c
 *          Title:  PCIe root complex initialization for LSI SAS3108
 *     Programmer:  Sandeep Darisala
 *  Creation Date:  March 13, 2012
 *
 * Description
 * ------------
 *
 *    This file is a sample application that uses the PCIe root complex
 *    library to initialize and operate the LSI SAS3108 chip in root complex
 *    mode.
 *-------------------------------------------------------------------------
 */

#include <pci.h>
#include "basetype.h"
#include "rootComplex.h"
#include "lsisas3108_pci.h"
#include <iopCommon.h>

PCIE_BRIDGE     pcieBridge[MAX_BRIDGES];
PCIE_DEVICE     pcieDevice[MAX_DEVICES];

U32        bridgeCount = 0;
U32        deviceCount = 0;
U32        maxBus      = 0;

static const PCIE_EMBEDDED_END_POINT_CONFIG_ADDR_DATA pciEmbEndPointConfig[PCIE_EMBEDDED_END_POINT_CONFIG_NUMBER] =
{
    {0xC, 0xFF08},
    {0x70, 0x2810},
    {0x78, 0x0},
    {0x118, 0x1E0},
    {0x64, 0x0},
    {0x64, 0x0},
    {0x4, 0x157}
};

static void     pcieConfigureEmbeddedEndPoint(void);
static void     pcieBusEnumeration(U8 Bus, S16 CurrentBridge);
static void     pcieSetupBARs(void);
static void     pcieSetMsixTableSizeAndBase(void);
static void     pcieConfigureBARs(void);
static void     pcieConfigurePIMWindow(void);
static void     pcieConfigurePOMWindow(void);
static void     pcieGetBARSizeAndAttributes(U32 DeviceId);
//static void     pcieConfigureMSIX(U32 DeviceId);
static void     pcieSetupIoRange(U32 DeviceId, U32 BaseAddr, U32 LimitAddr);
static void     pcieSetupMemoryRange(U32 DeviceId, U32 BaseAddr, U32 LimitAddr);

#if LSISAS3108_PCIE_DEBUG
static void     pcieDumpConfigurationHeader(void);
#endif  /* LSISAS3108_PCIE_DEBUG */

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pci_init_board
* SUMMARY:  U-Boot entry point for board specific PCI initialization.
* SCOPE:
*
*
* DESCRIPTION: This function is called by u-boot to perform board specific
*              PCI initialization. This function calls several other routines
*              to setup and initialize lsisas3108 chip for root complex
*              operation.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:    In the lsisas3108 configuration file, CONFIG_PCI
*                   must be defined for u-boot to include PCI support.
----------------------------------------------------------------------------*/
int pci_rc_init_board(void)
{
    int status;

    /* FIXME: work around for POM exception */
    iopiDiagConsolePrint0("Disable PLB Machine Check in L2 logic\n");
    MovToDcr(0x130, 0x2000);
    MovToDcr(0x9fe, 0x2004);

    /* Get link status */
    status = pcieRcIsLinkUp();

    /* If link is not up, perform link training to get the link up */
    if (status == -1)
    {
        /* Perform link width training */
        status = pcieRcPerformLinkWidthTraining(PCIE_LINK_WIDTH_4);
        if (status == -1)
        {
            iopiDiagConsolePrint0("Link width training failure. PCIe init terminated\n");
            return -1;
        }

        /* Perform link speed training */
        status = pcieRcPerformLinkSpeedTraining(PCIE_LINK_SPEED_5_0);
        if (status == -1)
        {
            iopiDiagConsolePrint1("Link speed(%d) training failure. PCIe init terminated\n", PCIE_LINK_SPEED_5_0);
            return -1;
        }
    }

    /*
     * Embedded end point needs to be configured before lsisas3108
     * root complex can generate commands.
     */
    pcieConfigureEmbeddedEndPoint();

    /* Perform PCIe enumeration */
    pcieBusEnumeration(0, -1);

    /* Setup BARs for the end points */
    pcieSetupBARs();

    /* Get MSI-X table sizes and offsets */
    pcieSetMsixTableSizeAndBase();

    /* Configure the BARs for the end points */
    pcieConfigureBARs();

    /* Program the PIM windows of the root complex */
    pcieConfigurePIMWindow();

    /* Program the POM windows of the root complex */
    pcieConfigurePOMWindow();

    /*
     * Enable the MSI-X capability of the root complex.
     * This is required for generating interrupts on inbound MSI-X messages.
     */
    pcieRcEnableMsixCapability();

#if LSISAS3108_PCIE_DEBUG
    pcieDumpConfigurationHeader();
#endif  /* LSISAS3108_PCIE_DEBUG */

    return maxBus;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieConfigureEmbeddedEndPoint
* SUMMARY:  Perform embedded end point configuration.
*
* SCOPE:
*
*
* DESCRIPTION:  This function configures the embedded end point of lsisas3108's
*               root complex.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieConfigureEmbeddedEndPoint(void)
{
    int configNum;

    for (configNum = 0; configNum < PCIE_EMBEDDED_END_POINT_CONFIG_NUMBER; configNum += 1)
    {
        pcieRcWriteBackDoor(pciEmbEndPointConfig[configNum].Addr, pciEmbEndPointConfig[configNum].Data);
    }

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieBusEnumeration
* SUMMARY:  Perfrom PCIe bus enumeration of the PCIe topology.
* SCOPE:
*
*
* DESCRIPTION:  This function performs a depth first scan of the PCIe
*               topology, geographically addressing the devices as
*               bus, device, function and configuring the devices
*               (bridges and end points) along the way.
*
*
* ARGUMENTS:
*   Bus           - The current bus under which devices will be scanned.
*   CurrentBridge - The current bridge under which devices will be scanned
*                   i.e. Bus is the secondary bus for CurrentBridge.
*
* RETURNS:  void
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieBusEnumeration(U8 Bus, S16 CurrentBridge)
{
    U8      device;
    U8      function;
    U8      bridge;
    U16     deviceId;
    U16     vendorId;
    U32     configHdrDword0;
    U8      headerType;

    /* Scan for all devices on the bus */
    for (device = 0; device < PCI_MAX_PCI_DEVICES; device += 1)
    {
        /*
         * The check below is based on the assumption that the upstream port
         * associated with the lsisas3108 root complex is bus 0 device 0 function 0
         * and all other devices are downstream of this port. So, when all devices
         * under the upstream port (bus 0 device 0 function 0) have been discovered,
         * complete the enumeration.
         */
        if (Bus == 0 && device != 1)
        {
            continue;
        }

        for (function = 0; function < 7; function++)
        {
            pcieRcCfgReadDword(Bus, device, function, PCI_VENDOR_ID, &configHdrDword0);
            vendorId = configHdrDword0 & 0x0000FFFF;
            deviceId = configHdrDword0 >> 16;

            if (vendorId != 0xFFFF)
            {
                pcieRcCfgReadByte(Bus, device, function, PCI_HEADER_TYPE, &headerType);

                iopiDiagConsolePrint4("Type %d bus %d device %d function %d ",headerType & 0x7F, Bus, device, function);
                iopiDiagConsolePrint2("vendor 0x%04x deviceId 0x%04x\n",vendorId, deviceId);

                if ((headerType & 0x80) != 0)
                {
                    iopiDiagConsolePrint0(" [multi-function device]\n");
                }

                if ((headerType & 0x7F) == PCI_HEADER_TYPE_BRIDGE)
                {
                    iopiDiagConsolePrint0(" [Bridge Device]\n");
                    bridge                            = bridgeCount;
                    pcieBridge[bridge].PrimaryBus     = Bus;
                    pcieBridge[bridge].SecondaryBus   = maxBus + 1;
                    pcieBridge[bridge].SubordinateBus = 255;
                    pcieBridge[bridge].DeviceNum      = deviceCount;

                    pcieBridge[bridge].IoBaseAddress   = 0xFFFFFFFF;
                    pcieBridge[bridge].IoLimitAddress  = 0;
                    pcieBridge[bridge].MemBaseAddress  = 0xFFFFFFFF;
                    pcieBridge[bridge].MemLimitAddress = 0;

                    pcieDevice[deviceCount].Bus            = Bus;
                    pcieDevice[deviceCount].Device         = device;
                    pcieDevice[deviceCount].Function       = function;
                    pcieDevice[deviceCount].HeaderType     = PCI_HEADER_TYPE_BRIDGE;
                    pcieDevice[deviceCount].ParentId       = (CurrentBridge == -1) ? CurrentBridge : pcieBridge[CurrentBridge].DeviceNum;
                    pcieDevice[deviceCount].BridgeNum      = bridge;

                    /* Update the bridge's configuration space for the primary bus */
                    pcieRcCfgWriteByte(Bus, device, function, 0x18, pcieBridge[bridge].PrimaryBus);

                    /* Update the bridge's configuration space for the secondary bus */
                    pcieRcCfgWriteByte(Bus, device, function, 0x19, pcieBridge[bridge].SecondaryBus);

                    /* Update the bridge's configuration space for the subordinate bus */
                    pcieRcCfgWriteByte(Bus, device, function, 0x1A, pcieBridge[bridge].SubordinateBus);

                    /* Probe BAR0/1 size */
                    pcieGetBARSizeAndAttributes(deviceCount);

                    bridgeCount += 1;
                    deviceCount += 1;
                    maxBus      += 1;

                    /* Scan for all devices under this bridge */
                    pcieBusEnumeration(pcieBridge[bridge].SecondaryBus, bridge);

                    /* All devices under this bridge scanned. Now, update its subordinate bus */
                    pcieBridge[bridge].SubordinateBus = maxBus;

                    /* Update the bridge's configuration space for the subordinate bus */
                    pcieRcCfgWriteByte(Bus, device, function, 0x1A, pcieBridge[bridge].SubordinateBus);
                }
                else if ((headerType & 0x7F) == PCI_HEADER_TYPE_NORMAL)
                {
                    iopiDiagConsolePrint0(" [End/Normal Device]\n");
                    pcieDevice[deviceCount].Bus        = Bus;
                    pcieDevice[deviceCount].Device     = device;
                    pcieDevice[deviceCount].Function   = function;
                    pcieDevice[deviceCount].HeaderType = PCI_HEADER_TYPE_NORMAL;
                    pcieDevice[deviceCount].ParentId   = pcieBridge[CurrentBridge].DeviceNum;

                    pcieGetBARSizeAndAttributes(deviceCount);

                    deviceCount += 1;
                    //maxBus      += 1;
                }
                else if ((headerType & 0x7F) == PCI_HEADER_TYPE_CARDBUS)
                {
                    /* Card bus not supported. Leaving this as a place holder */
                }
            }
            else
            {
                /* If function 0 is invalid then none of other functions can exist */
                if (function == 0)
                    break;
            }
        }
    }

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieSetupBARs
* SUMMARY:  Sets up Base Address Registers (BARs) for the PCIe end points.
* SCOPE:
*
*
* DESCRIPTION: This function sets up the BARs for the PCIe end points.
*              The BAR values set are later used by the pcieConfigureBARs()
*              function to actually program the end points BAR registers.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieSetupBARs(void)
{
    U32     deviceId;
    U8      barNum;
    U64     currentIoBaseAddr;
    U64     currentNonPrefetchMemBaseAddr;

    currentIoBaseAddr.Low               = PCIE_IO_BASE_ADDR_LOW;
    currentIoBaseAddr.High              = PCIE_IO_BASE_ADDR_HIGH;
    currentNonPrefetchMemBaseAddr.Low   = PCIE_NON_PREFETCHABLE_MEMORY_BASE_ADDR_LOW;
    currentNonPrefetchMemBaseAddr.High  = PCIE_NON_PREFETCHABLE_MEMORY_BASE_ADDR_HIGH;

    for (deviceId = 0; deviceId < deviceCount; deviceId++)
    {
        if (pcieDevice[deviceId].HeaderType == PCI_HEADER_TYPE_NORMAL ||
            pcieDevice[deviceId].HeaderType == PCI_HEADER_TYPE_BRIDGE)
        {
            barNum = 0;
            while (barNum < PCIE_TYPE0_HDR_MAX_BARS)
            {
                if (pcieDevice[deviceId].Bar[barNum].Attributes.Type == PCI_BASE_ADDRESS_SPACE_IO)
                {
                    /* Setup IO BAR */

                    if ((pcieDevice[deviceId].Bar[barNum].Size != 0) && (currentIoBaseAddr.Low < PCIE_IO_LIMIT_LOW))
                    {
                        pcieDevice[deviceId].Bar[barNum].Address.Low  = currentIoBaseAddr.Low;
                        pcieDevice[deviceId].Bar[barNum].Address.High = currentIoBaseAddr.High;

                        /* Setup upper ports to include this IO range */
                        pcieSetupIoRange(deviceId, currentIoBaseAddr.Low, (currentIoBaseAddr.Low + 0x1000) - 1);

                        /* Align IO base addresses on 4 KB boundary */
                        currentIoBaseAddr.Low += 0x1000;
                    }

                    barNum += 1;
                }
                else
                {
#if 0
                    if (pcieDevice[deviceId].Bar[barNum].Attributes.Prefetchable == 0)
                    {
                        /* Setup non-prefetchable memory BAR */

                        if ((pcieDevice[deviceId].Bar[barNum].Size != 0) && 
                            (currentNonPrefetchMemBaseAddr.Low < PCIE_NON_PREFETCHABLE_MEMORY_LIMIT))
                        {
                            pcieDevice[deviceId].Bar[barNum].Address.Low  = currentNonPrefetchMemBaseAddr.Low;
                            pcieDevice[deviceId].Bar[barNum].Address.High = currentNonPrefetchMemBaseAddr.High;

                            /* Setup upper ports to include this non-prefetchable memory range */
                            pcieSetupMemoryRange(deviceId, currentNonPrefetchMemBaseAddr.Low, 
                                                 (currentNonPrefetchMemBaseAddr.Low + 0x100000) - 1);

                            /* Align non-prefetchable addresses on 1 MB boundary */
                            currentNonPrefetchMemBaseAddr.Low += 0x100000;
                        }
                    }
                    else
                    {
                        /* TODO - Setup prefetchable memory BAR */
                    }
#else
                    /* [Cloud] arrange prefetch to non-prefetch to the same 32-bit window */
                    if ((pcieDevice[deviceId].Bar[barNum].Size != 0) && 
                        (currentNonPrefetchMemBaseAddr.Low < PCIE_NON_PREFETCHABLE_MEMORY_LIMIT))
                    {
                        U32 alignSize = 0x100000; /* 1MB aligned */

                        if (pcieDevice[deviceId].Bar[barNum].Size > alignSize)
                        {
                            alignSize = pcieDevice[deviceId].Bar[barNum].Size;
                        }

                        /* Align up if size > 1MB */
                        currentNonPrefetchMemBaseAddr.Low = (currentNonPrefetchMemBaseAddr.Low + (alignSize-1)) & ~(alignSize-1);

                        pcieDevice[deviceId].Bar[barNum].Address.Low  = currentNonPrefetchMemBaseAddr.Low;
                        pcieDevice[deviceId].Bar[barNum].Address.High = 0; /* force to 32-bit addressing */

                        /* Setup upper ports to include this non-prefetchable memory range */
                        pcieSetupMemoryRange(deviceId, currentNonPrefetchMemBaseAddr.Low, 
                                             (currentNonPrefetchMemBaseAddr.Low + alignSize) - 1);

                        /* Align non-prefetchable addresses on 1 MB boundary */
                        currentNonPrefetchMemBaseAddr.Low += 0x100000;
                    }
#endif

                    if (pcieDevice[deviceId].Bar[barNum].Attributes.AddrDecoding == 0)
                    {
                        barNum += 1;
                    }
                    else
                    {
                        /* For 64-bit address decode, defaulting upper 32-bits to zeros */
                        pcieDevice[deviceId].Bar[barNum + 1].Address.Low = 0;
                        barNum += 2;
                    }
                }
            }
        }
    }

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieSetMsixTableSizeAndBase
* SUMMARY:  Obtain MSI-X table sizes and offsets for the end points.
* SCOPE:
*
*
* DESCRIPTION: This function obtains the MSI-X table sizes and offsets for the
*              end points. It then calculates the MSI-X table base address
*              values for the end points. This information is useful later for
*              MSI-X configuration.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieSetMsixTableSizeAndBase(void)
{
    U8      bus;
    U8      device;
    U8      function;
    U8      msixCapabilityAddr;
    U8      bir;
    U32     tableSize;
    U32     tableOffset;
    U32     deviceId;

    for (deviceId = 0; deviceId < deviceCount; deviceId++)
    {
        if (pcieDevice[deviceId].HeaderType == PCI_HEADER_TYPE_NORMAL)
        {
            bus      = pcieDevice[deviceId].Bus;
            device   = pcieDevice[deviceId].Device;
            function = pcieDevice[deviceId].Function;

            msixCapabilityAddr = pcieRcFindCapability(bus, device, function, PCI_CAP_ID_MSIX);

            if (msixCapabilityAddr != 0)
            {
                pcieRcCfgReadWord(bus, device, function, msixCapabilityAddr + 2, (U16 *)&tableSize);
                pcieDevice[deviceId].MsixTableSize = tableSize & 0x07FF;

                pcieRcCfgReadDword(bus, device, function, msixCapabilityAddr + 4, &tableOffset);
                bir = tableOffset & 0x7;

                pcieDevice[deviceId].MsixTableBaseAddressLow = pcieDevice[deviceId].Bar[bir].Address.Low + 
                                                                (tableOffset & 0xFFFFFFF8);

                if (pcieDevice[deviceId].Bar[bir].Attributes.AddrDecoding == 2)
                {
                    pcieDevice[deviceId].MsixTableBaseAddressHigh = pcieDevice[deviceId].Bar[bir + 1].Address.High;
                }
                else
                {
                    pcieDevice[deviceId].MsixTableBaseAddressHigh = 0;
                }
            }
        }
    }

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieConfigureBARs
* SUMMARY:  Configure BAR registers of the end points.
* SCOPE:
*
*
* DESCRIPTION:  This function configures the BAR registers of the end points
*               using the values set by pcieSetupBARs() earlier.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:    pcieSetupBARs() should be completed.
*
----------------------------------------------------------------------------*/
static void pcieConfigureBARs(void)
{
    U32     deviceId;
    U8      bridgeId;
    U8      bus;
    U8      device;
    U8      function;
    U8      barNum;
    U8      ioBase;
    U8      ioLimit;
    U16     ioBaseUpper;
    U16     ioLimitUpper;
    U16     nonPrefetchableMemBase;
    U16     nonPrefetchableMemLimit;
    U32     configHdrDword1;

    for (deviceId = 0; deviceId < deviceCount; deviceId++)
    {
        bus      = pcieDevice[deviceId].Bus;
        device   = pcieDevice[deviceId].Device;
        function = pcieDevice[deviceId].Function;

        if (pcieDevice[deviceId].HeaderType == PCI_HEADER_TYPE_NORMAL)
        {
            barNum = 0;

            while (barNum < PCIE_TYPE0_HDR_MAX_BARS)
            {
                if (pcieDevice[deviceId].Bar[barNum].Attributes.AddrDecoding == 0)
                {
                    pcieRcCfgWriteDword(bus, device, function, 0x10 + (barNum * 4), 
                                         pcieDevice[deviceId].Bar[barNum].Address.Low);
                    barNum += 1;
                }
                else
                {
                    pcieRcCfgWriteDword(bus, device, function, 0x10 + (barNum * 4), 
                                         pcieDevice[deviceId].Bar[barNum].Address.Low);
                    pcieRcCfgWriteDword(bus, device, function, 0x10 + ((barNum + 1) * 4), 
                                         pcieDevice[deviceId].Bar[barNum].Address.High);
                    barNum += 2;
                }
            }
        }
        else if (pcieDevice[deviceId].HeaderType == PCI_HEADER_TYPE_BRIDGE)
        {
            /* Configure BAR0/1 */
            if (pcieDevice[deviceId].Bar[0].Attributes.AddrDecoding == 0)
            {
                pcieRcCfgWriteDword(bus, device, function, 0x10, pcieDevice[deviceId].Bar[0].Address.Low);
            }
            else
            {
                pcieRcCfgWriteDword(bus, device, function, 0x10, pcieDevice[deviceId].Bar[0].Address.Low);
                pcieRcCfgWriteDword(bus, device, function, 0x14, pcieDevice[deviceId].Bar[0].Address.High);
            }
            bridgeId = pcieDevice[deviceId].BridgeNum;

            /* Configure IO Base/Limit Registers */
            ioBase       = (pcieBridge[bridgeId].IoBaseAddress & 0x0000F000) >> 8;
            ioLimit      = (pcieBridge[bridgeId].IoLimitAddress & 0x0000F000) >> 8;
            ioBaseUpper  = pcieBridge[bridgeId].IoBaseAddress >> 16;
            ioLimitUpper = pcieBridge[bridgeId].IoLimitAddress >> 16;

            /* Configure non-prefetchable memory base/limit registers */
            nonPrefetchableMemBase  = (pcieBridge[bridgeId].MemBaseAddress & 0xFFF00000)  >> 16;
            nonPrefetchableMemLimit = (pcieBridge[bridgeId].MemLimitAddress & 0xFFF00000) >> 16;

            iopiDiagConsolePrint1("\nConfig Bar Write for Bridge id: %d\n",bridgeId);
            iopiDiagConsolePrint0("----------------------------------------------------\n");
            iopiDiagConsolePrint2("IoBase:     %08x IoBaseUpper:  %08x\n",ioBase,ioBaseUpper);
            iopiDiagConsolePrint2("IoLimit:    %08x IoLimitUpper: %08x\n",ioLimit,ioLimitUpper);
            iopiDiagConsolePrint2("MemBase:    %08x MemLimit:     %08x\n",pcieBridge[bridgeId].MemBaseAddress,
                                                                          pcieBridge[bridgeId].MemLimitAddress);
            iopiDiagConsolePrint2("NonPreBase: %08x NonPreLimit:  %08x\n",nonPrefetchableMemBase,nonPrefetchableMemLimit);
            
            /* Perform config writes to update the above registers */
            pcieRcCfgWriteByte(bus, device, function, 0x1C, ioBase);
            pcieRcCfgWriteByte(bus, device, function, 0x1D, ioLimit);
            pcieRcCfgWriteWord(bus, device, function, 0x30, ioBaseUpper);
            pcieRcCfgWriteWord(bus, device, function, 0x32, ioLimitUpper);
            pcieRcCfgWriteWord(bus, device, function, 0x20, nonPrefetchableMemBase);
            pcieRcCfgWriteWord(bus, device, function, 0x22, nonPrefetchableMemLimit);
            
            /* Perform config read to verify the above registers */
            pcieRcCfgReadByte(bus, device, function, 0x1C, &ioBase);
            pcieRcCfgReadByte(bus, device, function, 0x1D, &ioLimit);
            pcieRcCfgReadWord(bus, device, function, 0x30, &ioBaseUpper);
            pcieRcCfgReadWord(bus, device, function, 0x32, &ioLimitUpper);
            pcieRcCfgReadWord(bus, device, function, 0x20, &nonPrefetchableMemBase);
            pcieRcCfgReadWord(bus, device, function, 0x22, &nonPrefetchableMemLimit);
            
            iopiDiagConsolePrint1("\nConfig Bar Read for Bridge id: %d\n",bridgeId);
            iopiDiagConsolePrint0("----------------------------------------------------\n");
            iopiDiagConsolePrint2("IoBase:     %08x IoBaseUpper:  %08x\n",ioBase,ioBaseUpper);
            iopiDiagConsolePrint2("IoLimit:    %08x IoLimitUpper: %08x\n",ioLimit,ioLimitUpper);
            iopiDiagConsolePrint2("MemBase:    %08x MemLimit:     %08x\n",pcieBridge[bridgeId].MemBaseAddress,
                                                                          pcieBridge[bridgeId].MemLimitAddress);
            iopiDiagConsolePrint2("NonPreBase: %08x NonPreLimit:  %08x\n\n\n",nonPrefetchableMemBase,nonPrefetchableMemLimit);
        }

        /* Enable BusMaster and IO and memory address space decoding in the device's command register */
        pcieRcCfgReadDword(bus, device, function, 0x4, &configHdrDword1);
        configHdrDword1 |= 0x00000007;
        pcieRcCfgWriteDword(bus, device, function, 0x4, configHdrDword1);
    }
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieConfigurePIMWindow
* SUMMARY:  PIM window configuration.
* SCOPE:
*
*
* DESCRIPTION:  This function configures the PIM windows of the lsisas3108
*               root complex.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieConfigurePIMWindow(void)
{
    PCIE_RC_CONFIG_PIM_PARAMS   PIMConfigParams;

    /*
     * Configure PIM0 window.
     * Set size to 16 KB. Enable PIM0 window, Messaging Unit, and MSI-X.
     * First 8 KB points to Messaging Unit and second 8 KB points to MSI-X region.
     */
    PIMConfigParams.PIMSizeLow                   = PIM0_WINDOW_SIZE;
    PIMConfigParams.PIMSizeHigh                  = 0xFFFFFFFF;
    PIMConfigParams.PIMAttributes.MSIXDisable    = 0;
    PIMConfigParams.PIMAttributes.MUEn           = 1;
    PIMConfigParams.PIMAttributes.PIMEnable      = 1;
    PIMConfigParams.PIMAttributes.PIMShell       = 0;
    PIMConfigParams.PIMAttributes.PrefetchEnable = 0;
    PIMConfigParams.PCIeAddressLow               = PIM0_PCIE_BASE_ADDRESS_LOW;
    PIMConfigParams.PCIeAddressHigh              = PIM0_PCIE_BASE_ADDRESS_HIGH;
    PIMConfigParams.PIMId                        = 0;
    pcieRcConfigurePIM(&PIMConfigParams);

    /*
     * Configure PIM1 window.
     * Map PIM1 window to DDR memory. Set size to 2 GB and enable PIM1 window.
     */
    PIMConfigParams.PIMSizeLow                   = PIM1_WINDOW_SIZE;
    PIMConfigParams.PIMSizeHigh                  = 0xFFFFFFFF;
    PIMConfigParams.PIMAttributes.MSIXDisable    = 0;
    PIMConfigParams.PIMAttributes.MUEn           = 0;
    PIMConfigParams.PIMAttributes.PIMEnable      = 1;
    PIMConfigParams.PIMAttributes.PIMShell       = 0;
    PIMConfigParams.PIMAttributes.PrefetchEnable = 0;
    PIMConfigParams.PLBAddressLow                = 0;
    PIMConfigParams.PLBAddressHigh               = 0;
    PIMConfigParams.PCIeAddressLow               = PIM1_PCIE_BASE_ADDRESS_LOW;
    PIMConfigParams.PCIeAddressHigh              = PIM1_PCIE_BASE_ADDRESS_HIGH;
    PIMConfigParams.PIMId                        = 1;
    pcieRcConfigurePIM(&PIMConfigParams);

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieConfigurePOMWindow
* SUMMARY:  POM window configuration.
* SCOPE:
*
*
* DESCRIPTION:  This function configures the POM windows of the lsisas3108
*               root complex.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:
*
* NOTE: Currently, only POM0 window is configured. POM1 window configuration
*       can be added if required.
----------------------------------------------------------------------------*/
static void pcieConfigurePOMWindow(void)
{
    PCIE_RC_CONFIG_POM_PARAMS   POMConfigParams;

    /* Configure POM0 window */
    POMConfigParams.PLBAddressLow   = POM0_PLB_START_ADDRESS;
    POMConfigParams.PLBAddressHigh  = 0x70;
    POMConfigParams.PCIeAddressLow  = POM0_PLB_START_ADDRESS;
    POMConfigParams.PCIeAddressHigh = 0;
    POMConfigParams.POMSize         = POM0_WINDOW_SIZE;
    POMConfigParams.SIL             = 0;
    POMConfigParams.POMId           = 0;
    pcieRcConfigurePOM(&POMConfigParams);

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieGetBARSizeAndAttributes
* SUMMARY:  Obtain BAR properties of an end point.
* SCOPE:
*
*
* DESCRIPTION: This function is used by PCIe enumeration to obtain BAR
*              properties of an end point.
*
* ARGUMENTS:
*   DeviceId - device identifier of an end point.
*
* RETURNS:  None
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieGetBARSizeAndAttributes(U32 DeviceId)
{
    U8      bus;
    U8      device;
    U8      function;
    U8      barNum;
    U32     barAddress;
    U32     configHdrDword;

    bus      = pcieDevice[DeviceId].Bus;
    device   = pcieDevice[DeviceId].Device;
    function = pcieDevice[DeviceId].Function;
    barNum   = 0;

    while (barNum < PCIE_TYPE0_HDR_MAX_BARS)
    {
        barAddress = PCI_BASE_ADDRESS_0 + (barNum * 4);

        pcieRcCfgReadDword(bus, device, function, barAddress, &configHdrDword);

        if ((configHdrDword & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_MEMORY)
        {
            /* Memory BAR */
            pcieDevice[DeviceId].Bar[barNum].Attributes.Type = 0;
        }
        else if ((configHdrDword & PCI_BASE_ADDRESS_SPACE) == PCI_BASE_ADDRESS_SPACE_IO)
        {
            /* IO BAR */
            pcieDevice[DeviceId].Bar[barNum].Attributes.Type = 1;
        }

        if ((configHdrDword & PCI_BASE_ADDRESS_MEM_TYPE_MASK) == PCI_BASE_ADDRESS_MEM_TYPE_32)
        {
            /* 32-bit address decoding */
            pcieDevice[DeviceId].Bar[barNum].Attributes.AddrDecoding = 0;
        }
        else if ((configHdrDword & PCI_BASE_ADDRESS_MEM_TYPE_MASK) == PCI_BASE_ADDRESS_MEM_TYPE_64)
        {
            /* 64-bit address decoding */
            pcieDevice[DeviceId].Bar[barNum].Attributes.AddrDecoding = 2;
        }

        if ((configHdrDword & PCI_BASE_ADDRESS_MEM_PREFETCH) == 0)
        {
            /* Non-prefetchable */
            pcieDevice[DeviceId].Bar[barNum].Attributes.Prefetchable = 0;
        }
        else
        {
            /* Prefetchable */
            pcieDevice[DeviceId].Bar[barNum].Attributes.Prefetchable = 1;
        }

        /* Get size */
        if (pcieDevice[DeviceId].Bar[barNum].Attributes.AddrDecoding == 0)
        {
            pcieRcCfgWriteDword(bus, device, function, barAddress, 0xFFFFFFFF);
            pcieRcCfgReadDword(bus, device, function, barAddress, &configHdrDword);
            pcieDevice[DeviceId].Bar[barNum].Size = ~(configHdrDword & 0xFFFFFFF0) + 1;

            barNum += 1;
        }
        else if (pcieDevice[DeviceId].Bar[barNum].Attributes.AddrDecoding == 2)
        {
            pcieRcCfgWriteDword(bus, device, function, barAddress, 0xFFFFFFFF);
            pcieRcCfgWriteDword(bus, device, function, barAddress + 4, 0xFFFFFFFF);

#if 1
            pcieRcCfgReadDword(bus, device, function, barAddress, &configHdrDword);
            pcieDevice[DeviceId].Bar[barNum].Size = ~(configHdrDword & 0xFFFFFFF0) + 1;
#else       /* Need to check 64-bit support before enabling this code */
            configHdrDword     = pcieRcCfgReadDword(bus, device, function, barAddress);
            configHdrDwordNext = pcieRcCfgReadDword(bus, device, function, barAddress + 4);
            /* Size should be made 64-bits */
            pcieDevice[DeviceId].Bar[barNum].Size =
                ~(((configHdrDwordNext << 32) | configHdrDword) & PCI_BASE_ADDRESS_MEM_MASK) + 1;
#endif

            barNum += 2;
        }
        if (pcieDevice[DeviceId].HeaderType == PCI_HEADER_TYPE_BRIDGE)
            break;
    }

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieConfigureMSIX
* SUMMARY:  MSI-X vector configuration.
* SCOPE:
*
*
* DESCRIPTION:  This function is used to serve as a simple demonstration
*               of how lsisas3108 root complex can configure a MSI-X
*               vector entry for a given end point. Note that the SDK
*               performs MSI-X configuration in the Linux kernel and so
*               does not use this functionality provided in the u-boot.
*
* ARGUMENTS:
*   DeviceId -  the device identifier for which MSI-X table needs
*               configuration.
*
* RETURNS:  void
*
*
* PREREQUISITES:
*
* NOTE: This function configures a single vector entry (vector 0)
*       for the given device. If more vectors/more devices need to be
*       configured, then this function needs to be modified accordingly.
----------------------------------------------------------------------------*/
#if 0
static void pcieConfigureMSIX(U32 DeviceId)
{
    U32                     barAddress;
    U32                     pim0Size;
    PCIE_RC_MSIX_MESSAGE    msixMessage;

    /* MSI-X region steered through PIM0 window */

    /* Get BAR1 address */
    pcieRcReadBackDoor(PCI_BASE_ADDRESS_1, &barAddress);

    /* Get PIM0 window size */
    pim0Size   = PIM0_WINDOW_SIZE;

    /* Inbound MSIX region base = PIM0 base + PIM0 size - 8 KB */
    msixMessage.LowAddress    = (barAddress + pim0Size) - (MSIX_REGION_START_FROM_PIM0_TOP);
    pcieRcReadBackDoor(PCI_BASE_ADDRESS_2, &msixMessage.HighAddress);
    msixMessage.Data          = 1;
    msixMessage.VectorControl = 0;

    pcieRcConfigureMSIX(pcieDevice[DeviceId].Bus,
                        pcieDevice[DeviceId].Device,
                        pcieDevice[DeviceId].Function,
                        0,
                        &msixMessage
                        );
    return;
}
#endif
/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieSetupIoRange
* SUMMARY:  Setup IO address ranges for upstream bridges.
* SCOPE:
*
*
* DESCRIPTION: This function is used to setup IO address ranges for
*              upstream ports so that IO transactions can be routed
*              correctly.
*
* ARGUMENTS:
*   DeviceId  - device address of an end point
*   BaseAddr  - base address of the IO address range
*   LimitAddr - limiting address of the IO address range
*
* RETURNS:  None
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieSetupIoRange(U32 DeviceId, U32 BaseAddr, U32 LimitAddr)
{
    S16 parentDeviceId;
    U8      bridgeId;

    if (pcieDevice[DeviceId].HeaderType == PCI_HEADER_TYPE_BRIDGE)
        parentDeviceId = DeviceId; /* Start from self */
    else
        parentDeviceId = pcieDevice[DeviceId].ParentId;

    while (parentDeviceId != -1)
    {
        bridgeId = pcieDevice[parentDeviceId].BridgeNum;

        /* Lower base address range if required */
        if (pcieBridge[bridgeId].IoBaseAddress > BaseAddr)
        {
            pcieBridge[bridgeId].IoBaseAddress = BaseAddr;
        }

        /* Increase limit address range if required */
        if (pcieBridge[bridgeId].IoLimitAddress < LimitAddr)
        {
            pcieBridge[bridgeId].IoLimitAddress = LimitAddr;
        }

        parentDeviceId = pcieDevice[parentDeviceId].ParentId;
    }

    return;
}

/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieSetupMemoryRange
* SUMMARY:  Setup memory address range for upstream bridges.
* SCOPE:
*
* DESCRIPTION:  This function is used to setup memory address ranges for
*              upstream ports so that memory transactions can be routed
*              correctly.
*
* ARGUMENTS:
*   DeviceId  - device address of an end point
*   BaseAddr  - base address of the memory address range
*   LimitAddr - limiting address of the memory address range
*
* RETURNS:  void
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieSetupMemoryRange(U32 DeviceId, U32 BaseAddr, U32 LimitAddr)
{
    S16 parentDeviceId;
    U8      bridgeId;

    if (pcieDevice[DeviceId].HeaderType == PCI_HEADER_TYPE_BRIDGE)
        parentDeviceId = DeviceId; /* Start from self */
    else
        parentDeviceId = pcieDevice[DeviceId].ParentId;

    while (parentDeviceId != -1)
    {
        bridgeId = pcieDevice[parentDeviceId].BridgeNum;

        /* Lower base address range if required */
        if (pcieBridge[bridgeId].MemBaseAddress > BaseAddr)
        {
            pcieBridge[bridgeId].MemBaseAddress = BaseAddr;
        }

        /* Increase limit address range if required */
        if (pcieBridge[bridgeId].MemLimitAddress < LimitAddr)
        {
            pcieBridge[bridgeId].MemLimitAddress = LimitAddr;
        }

        parentDeviceId = pcieDevice[parentDeviceId].ParentId;
    }

    return;
}

#if LSISAS3108_PCIE_DEBUG
/*----------------------------------------------------------------------------
* PROCEDURE
*
* NAME:     pcieDumpConfigurationHeader
* SUMMARY:  Dump configuration space header for all devices discovered.
* SCOPE:
*
*
* DESCRIPTION:  This is a helper function to dump the configuration space
*               header for all devices that have been discovered. Used for
*               debug purpose.
*
* ARGUMENTS:    None
*
* RETURNS:      void
*
* PREREQUISITES:
*
----------------------------------------------------------------------------*/
static void pcieDumpConfigurationHeader(void)
{
    U32     deviceId;
    U8      bus;
    U8      device;
    U8      function;
    U8      dwordOffset;
    U32     configHdrDword;

    for (deviceId = 0; deviceId < deviceCount; deviceId++)
    {
        bus      = pcieDevice[deviceId].Bus;
        device   = pcieDevice[deviceId].Device;
        function = pcieDevice[deviceId].Function;

        iopiDiagConsolePrint3("Configuration header for bus %d device %d function %d\n", bus, device, function);
        for (dwordOffset = 0; dwordOffset < 16; dwordOffset++)
        {
            pcieRcCfgReadDword(bus, device, function, dwordOffset * 4, &configHdrDword);
            iopiDiagConsolePrint2("%04d : 0x%08x\n", dwordOffset, configHdrDword);
        }
        iopiDiagConsolePrint0("\n");
    }

    return;
}
#endif  /* LSISAS3108_PCIE_DEBUG */

